""" Reads out meta data of PDFs """

import decimal
import logging
import re
from datetime import datetime
from enum import Enum
import unicodedata

import cv2 as cv
import numpy as np
import pdfplumber
from sklearn.cluster import KMeans

from articles.serializers import write_article_to_db

logger = logging.getLogger(__name__)


class Category(Enum):
    """ Given categories from pressreview """
    UP = 'Universität Potsdam'
    UP_SOZ = 'Universität Potsdam in den sozialen Netzen'
    BER = 'Hochschulen Berlin'
    BRB = 'Hochschulen Brandenburg'
    IT = 'IT im Hochschulbereich'
    NOCAT = 'No category'


class Article:
    """ Article Object which should be later written in database """

    def __init__(self):
        self.title = ''
        self.reach = 0
        self.medium = ''
        self.tags = []
        self.date_of_publication = datetime.strptime('01.01.1970', '%d.%m.%Y')
        self.text = ''
        self.pages = 0
        self.wrongly_parsed = False

    def __repr__(self):
        return self.to_output()

    def __str__(self):
        return self.to_output()

    def to_output(self):
        """ to string function for repr() and str()"""
        return 'title:' + self.title \
               + ', reach:' + str(self.reach) \
               + ', medium:' + self.medium \
               + ', tags:' + ', '.join(self.tags) \
               + ', date:' + str(self.date_of_publication) \
               + ', text:' + str((self.text[:20] + '..') if len(self.text[:20]) > 75 else self.text[:20]) \
               + ', pages:' + str(self.pages) \
               + ', wrongly parsed:' + str(self.wrongly_parsed)


# pylint: disable=too-many-locals
def extract_infos(article_file, temp_dir, dir_path):
    """ extracts meta information from inhaltsverzeichnis """
    with pdfplumber.open(dir_path + article_file.upload_file) as pdf:
        logger.debug("in extracting infos from %s", article_file.upload_file)
        ihvz = {}
        inhaltsverzeichnis_text = pdf.pages[0].extract_text(x_tolerance=3, y_tolerance=3)
        inhaltsverzeichnis_fortsetzung = pdf.pages[1].extract_text(x_tolerance=3, y_tolerance=3)

        # Inhaltsverzeichnis auslesen
        if 'Inhaltsverzeichnis Fortsetzung' in inhaltsverzeichnis_fortsetzung:
            # sometimes are sooo many article that the ihvz is about two pages
            inhaltsverzeichnis_text += inhaltsverzeichnis_fortsetzung
        iterate_content_overview(ihvz, inhaltsverzeichnis_text)

        # Info Box auf den Artikelseiten
        logger.debug('extract info box of %s', article_file.upload_file)
        extract_info_box(pdf, ihvz, len(ihvz) * 2)

        # extract pages
        logger.debug('extract pages of %s', article_file.upload_file)
        extract_pages(ihvz, pdf)

        for page_number, article in ihvz.items():
            # Empty strings are "falsy"
            if page_number:
                for page_counter in range(article.pages):
                    extract_text(article, pdf, int(page_number) + page_counter - 1, temp_dir)

            if article.date_of_publication == datetime.strptime('01.01.1970', '%d.%m.%Y') or article.date_of_publication > datetime.today():
                # fallback to date in filename e.g. Pressespiegel_20210614.pdf
                datestring = str(article_file.upload_file).split('_')[1].split('.')[0]
                date_of_pub_ = datetime.strptime(datestring, "%Y%m%d")
                article.date_of_publication = date_of_pub_
                # insert pressespiegel as fallback medium
                quelle_ = f'pressespiegel ({datestring})'
                article.medium = quelle_
                article.wrongly_parsed = True

            if article.text == '':
                article.wrongly_parsed = True

        logger.debug(ihvz)

        # Varibales for logger info summary
        article_count = len(ihvz.values())
        failed_articles = 0

        # write all articles to db
        for value in ihvz.values():
            # wanna change the dict so we have to copy it
            value_dict = value.__dict__.copy()
            value_dict.pop('medium')
            if value_dict['wrongly_parsed']:
                failed_articles = failed_articles + 1
            write_article_to_db(source_file=article_file, medium_type='Zeitung', medium_name=value.medium, **value_dict)

        article_file.extraction_complete = True
        article_file.save()
        logger.info('Found %i articles (parsing fail %i) in file %s', article_count, failed_articles, article_file.upload_file)


def extract_pages(ihvz, pdf):
    """ calculates how many pages each article has """
    keys = sorted([int(key) for key in ihvz], key=int, reverse=False)
    netto_pages = [b - a for a, b in zip(keys, keys[1:] + [len(pdf.pages)])]
    for k, page in zip(keys, netto_pages):
        ihvz[str(k)].pages = page


def iterate_content_overview(ihvz, inhaltsverzeichnis_text):
    """ extracts information from the ihvz """
    ihvz_started = False
    two_liner = False
    category = Category.NOCAT
    prefix = ''
    for line in inhaltsverzeichnis_text.split("\n"):
        if "Inhaltsverzeichnis" in line:
            ihvz_started = True
            continue
        if ihvz_started and re.match(r'.* \(.*, .*\)[ ]?\d*', line) and not two_liner:
            title = line.split(' (')[0]
            page_number = line.split(' ')[-1]
            if ')' in str(page_number):
                page_number = page_number.split(')')[-1]
            create_article(ihvz, page_number, title, category)
            two_liner = False
        elif ihvz_started and re.match(r'.* \d*$', line) and not two_liner:
            # two_liner means that one item has a long title so they need two rows
            two_liner = True
            prefix = line
        elif ihvz_started and two_liner:
            # page number needs to be separated before concat because the number is always in the first row
            page_number = prefix.rsplit(' ', maxsplit=1)[-1]
            concat_line = prefix.replace(page_number, '') + line
            title = concat_line.split(' (')[0]
            create_article(ihvz, page_number, title, category)
            two_liner = False
        elif ihvz_started:
            is_cat = False
            for cat in Category:
                if cat.value in line:
                    category = cat
                    is_cat = True
            if not is_cat:
                logger.warning('not found:' + line)


def extract_text(article, pdf, page_number, temp_dir):
    """ extracts the actual text from the pages """
    if page_number < 0:
        return

    # remove special characters in title
    article_name = ''.join(e for e in article.title if e.isalnum())
    article_name = article_name.lower() \
        .replace('ü', 'ue') \
        .replace('ä', 'ae') \
        .replace('ö', 'oe') \
        .replace('ß', 'sz') \
        .replace('é', 'e') \
        .replace('è', 'e')
    article_name_path = temp_dir + '\\' + article_name + str(page_number)
    file_path = article_name_path + '.png'

    pdf_page = pdf.pages[page_number]
    plumber_im = pdf_page.to_image()
    found_rects = []
    to_cluster = []

    plumber_im.reset().draw_rects(pdf_page.chars)
    plumber_im.save(file_path, format="PNG")

    dilate_contours(file_path, found_rects, to_cluster)

    fitted_rects = cluster_rects(found_rects, plumber_im, to_cluster)
    plumber_im.save(article_name_path + "_clustered.png", format="PNG")
    cropped_text = ''
    for rect in fitted_rects:
        pdf_cropped_text = pdf_page.crop(rect).extract_text(x_tolerance=1, y_tolerance=3)
        if isinstance(pdf_cropped_text, str):
            cropped_text += pdf_cropped_text.replace('-\n', '')
    ' '.join(cropped_text.split('\n'))

    article.text += cropped_text


def cluster_rects(found_rects, plumber_im, to_cluster):
    """ clustering rects to find larger ones which could be the searched text boxes """
    kmeans = KMeans(n_clusters=2)
    np_heights = np.array(to_cluster)
    kmeans.fit(np_heights)
    fitted_rects = []
    # the center of the text boxes has to be larger
    # this fact is used to determine which label is for our text boxes
    good_label = 1 if kmeans.cluster_centers_[0][1] < kmeans.cluster_centers_[1][1] else 0
    plumber_im.reset()
    for rect, label in zip(found_rects, kmeans.labels_):
        if label == good_label:
            plumber_im.draw_rect(rect, stroke="green")
            fitted_rects.append(rect)
        else:
            plumber_im.draw_rect(rect, stroke="red")
    return fitted_rects


def dilate_contours(file_path, found_rects, to_cluster):
    """ find contours surrounding text boxes """
    # https://stackoverflow.com/questions/57249273/how-to-detect-paragraphs-in-a-text-document-image-for-a-non-consistent-text-stru
    # Load image, grayscale, Gaussian blur, Otsu's threshold
    image = cv.imread(file_path)
    gray = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
    blur = cv.GaussianBlur(gray, (7, 7), 0)
    thresh = cv.threshold(blur, 0, 255, cv.THRESH_BINARY_INV + cv.THRESH_OTSU)[1]
    # Create rectangular structuring element and dilate
    kernel = cv.getStructuringElement(cv.MORPH_RECT, (5, 5))
    dilate = cv.dilate(thresh, kernel, iterations=1)
    # Find contours and draw rectangle
    contours = cv.findContours(dilate, cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE)
    contours = contours[0] if len(contours) == 2 else contours[1]
    for contour in contours:
        x_value, y_value, width, height = cv.boundingRect(contour)
        cv.rectangle(image, (x_value, y_value), (x_value + width, y_value + height), (36, 255, 12), 2)
        found_rects.append((x_value, y_value, x_value + width, y_value + height))
        to_cluster.append((width, height))


def create_article(ihvz, page_number, title, category):
    """ creates an article object based on inhaltsverzeichnis """
    article = Article()
    article.title = title
    article.tags.append(category.value)
    if not page_number:
        # it couldn't find the page_number, so we assume it from the last article
        article.wrongly_parsed = True
        pages = sorted([int(k) for k in ihvz.keys()], key=int, reverse=False)
        if len(pages) == 0:
            page_number = '1'
        else:
            page_number = str(pages[-1] + 1)
    ihvz[page_number] = article


def extract_info_box(pdf, ihvz, expected_rects):
    """ extract the meta data from the info boxes """
    rects = pdf.rects
    actual_rects = 0
    for rect in rects:
        # Filter rects for the info box at the top of an article
        # the position of rects is a range because they are not everytime at the same place
        if (decimal.Decimal('40') <= rect['x0'] <= decimal.Decimal('45')) \
                and (decimal.Decimal('425') <= rect['x1'] <= decimal.Decimal('441')):
            actual_rects += 1
            page_number_ = rect['page_number']
            text = pdf.pages[page_number_ - 1].crop(
                (rect['x0'], rect['top'], rect['x1'], rect['bottom'])).extract_text()
            text = unicodedata.normalize("NFKC", text)
            if str(page_number_) not in ihvz:
                # this could mean that we couldn't crop the right page_number out of the ihvz. You need to correct it
                # manually in the database
                logger.warning('page %s not found', str(page_number_))
                continue
            article_ = ihvz[str(page_number_)]
            # get source, date of publication and page_number
            if re.match(r'Quelle: .* vom .*', text):
                splitted_quelle = text.split("Quelle: ")[1].split(" vom ")
                quelle_ = splitted_quelle[0]
                article_.medium = quelle_
                date_of_pub_ = datetime.strptime(splitted_quelle[1].split(', Seite')[0], "%d.%m.%Y")
                article_.date_of_publication = date_of_pub_
            # get reach
            elif re.match('Auflage:.*', text):
                reach_ = text.split('Auflage:')[1].strip()
                article_.reach = reach_

    if actual_rects != expected_rects:
        # if there are 0 rects found it is likely that the boxes are out of range
        logger.warning('expected and actual rects don\'t match... found: %s , expected: %s ', str(actual_rects),
                       str(expected_rects))
