""" Test class for article module """
import datetime
import logging
from unittest import mock

from django.db.models.signals import post_save
from django.test import TransactionTestCase

from articles.models import Article, Files, Medium, MediumType, User, Tag, ParentTag
from articles.serializers import write_article_to_db, ArticleSerializer
from articles.definitions import MediaRegionality, MediaOverviewType

logger = logging.getLogger(__name__)


class CreateArticlesWithTagsTestCase(TransactionTestCase):
    """
    This is a parent class for test cases that require an article to be in the database.
    When inherited it writes an article with two tags into the database and marks these tags as favorites for User 1.
    """

    def setUp(self):
        """
        Creates two tags 'Test1' and 'Test2' and a parent tag 'Parent1',
        adds these tags to User 1 and then writes an exemplary article 'Testartikel' to the test database.
        """
        tag_1 = Tag(name="Test1")
        tag_1.save()
        tag_2 = Tag(name="Test2")
        tag_2.save()
        parent = ParentTag(name="Parent1")
        parent.save()
        tag_1.parent.add(parent)
        tag_2.parent.add(parent)

        # add tags to User 1
        if User.objects.filter(id_tag=1).count() == 1:
            user = User.objects.get(id_tag=1)
            for tag in Tag.objects.all():
                logger.info(tag)
                user.favorite_tags.add(tag)
        else:
            logger.warning("No user 1 is there")

        with mock.patch('articles.signals.extract_information_from_file', autospec=True) as mocked_post_save:
            post_save.connect(mocked_post_save, sender=Files, dispatch_uid='extract_information_from_file')
            Files.objects.create(upload_file='testfile.pdf', upload_date='18.11.2021')
            self.file = Files.objects.get(upload_file='testfile.pdf')
            input_data = {
                'medium_name': 'MAZ',
                'medium_type': MediaRegionality.REGIONAL.value,
                'name': 'Testartikel',
                'source_file': self.file,
                'reach': 1000,
                'date_of_publication': datetime.datetime.today(),
                'text': 'Dies ist ein beispielhafter Beispieltext',
                'pages': 2,
                'wrongly_parsed': False,
                'tags': [tag_1.name, tag_2.name]
            }
            write_article_to_db(**input_data)


class OverviewTest(CreateArticlesWithTagsTestCase):
    """
    This class contains test for the Overview REST API
    defined in views.py > 'class OverviewList(APIView)'.
    """

    def test_create_user_app_conf(self):
        """
        Tests if the User 1 was created and whether the tags
        created in the parent class have been correctly marked as favorite_tags for User 1.
        """
        logger.info("started OverviewTest.test_create_user_app_conf")
        user = User.objects.get(id_tag=1)
        self.assertTrue(user.activated)
        self.assertEqual(user.favorite_tags.all().count(), 2)

    def test_overview_api_wo_user(self):
        """
        Tests the API without specifying a user in the query.
        The API should default to the user with id_tag = 1 and return an object for the test tags defined in the parent class.
        """
        logger.info("started OverviewTest.test_overview_api_wo_user")
        response = self.client.get('/articles/overview')
        self.assertEqual(response.status_code, 200)
        today_str = datetime.datetime.today().strftime('%Y-%m-%d')
        self.assertJSONEqual(str(response.content, encoding='utf8'), [
            {"name": "Test1", "article_name": "Testartikel", "medium_name": "maz",
             "date_of_publication": today_str,
             "reach": "1000"
             },
            {"name": "Test2", "article_name": "Testartikel", "medium_name": "maz",
             "date_of_publication": today_str,
             "reach": "1000"}])

    def test_overview_api_with_user(self):
        """
        Tests the API with a specific user id passed in the query.
        Creates a new user with id_tag = 2 and adds a tag 'Test2' to the users favorite_tags.
        The API should then only return an object for the tag 'Test2' (and not 'Test1', since it's not a favorite of User 2).
        """
        logger.info("started OverviewTest.test_overview_api_with_user")
        today_str = datetime.datetime.today().strftime('%Y-%m-%d')
        user = User(id_tag=2)
        user.save()
        tag2 = Tag.objects.get(name="Test2")
        user.favorite_tags.add(tag2)
        user.save()
        response = self.client.get('/articles/overview?user=' + str(user.id_tag))
        self.assertJSONEqual(str(response.content, encoding='utf8'), [
            {"name": tag2.name, "article_name": "Testartikel", "medium_name": "maz",
             "date_of_publication": today_str,
             "reach": "1000"}
        ])

    def test_overview_with_wrong_uid(self):
        """
        Tests how the API handles a request with a user id that doesn't exist.
        The API should return a response with status_code 400 (Bad Request).
        """
        logger.info("started OverviewTest.test_overview_with_wrong_uid")
        response = self.client.get('/articles/overview?user=2')
        self.assertEqual(response.status_code, 400)


class RecentArticleTest(CreateArticlesWithTagsTestCase):
    """
    This class contains test for the RecentArticlesForMedium REST API
    defined in views.py > 'class RecentArticlesForMedium(APIView)'.
    """

    def test_recent_articles_wo_user(self):
        """
        Tests the API case where neither a user nor a tag is specified in the query.
        Should return all articles from the medium 'MAZ' in the test database.
        """
        logger.info("started RecentArticleTest.test_recent_articles_wo_user")
        response = self.client.get('/articles/articles/medium/recent?mediaOverviewType=' + MediaOverviewType.ALLTAGS.value + '&medium_name=MAZ')
        today_str = datetime.datetime.today().strftime('%Y-%m-%d')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Testartikel', 'date_of_publication': today_str, 'text': 'Dies ist ein beispielhafter Beispieltext'}])

    def test_recent_articles_usertags(self):
        """
        Tests the API case where only articles with the users favorite tags are taken into account.
        Creates user_2, and tag_3 and adds tag_3 to user_2's favorite tags. Writes a new article using tag_3 to the database.
        The API response should only contain the newly created article since its the only article tagged with tag_3.
        """
        logger.info("started RecentArticleTest.test_recent_articles_usertags")
        user_2 = User(id_tag=2)
        user_2.save()
        tag_3 = Tag(name="Test3")
        tag_3.save()
        user_2.favorite_tags.add(tag_3)
        user_2.save()
        today_str = datetime.datetime.today().strftime('%Y-%m-%d')

        self.file = Files.objects.get(upload_file='testfile.pdf')
        input_data = {
            'medium_name': 'MAZ',
            'medium_type': MediaRegionality.REGIONAL.value,
            'name': 'Testartikel-2',
            'source_file': self.file,
            'reach': 1000,
            'date_of_publication': datetime.datetime.today(),
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': False,
            'tags': [tag_3.name]
        }
        write_article_to_db(**input_data)

        response = self.client.get('/articles/articles/medium/recent?mediaOverviewType=' + MediaOverviewType.USERTAGS.value + '&medium_name=MAZ&user=2')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Testartikel-2', 'date_of_publication': today_str, 'text': 'Dies ist ein beispielhafter Beispieltext'}])

    def test_recent_articles_querytag(self):
        """
        Tests the API case where a specific tag ('Test1') is passed as a query.
        Should return all articles from the medium 'MAZ' with the tag 'Test1' in the test database.
        """
        logger.info("started RecentArticleTest.test_recent_articles_querytag")
        tag = Tag.objects.get(name="Test1")
        response = self.client.get('/articles/articles/medium/recent?mediaOverviewType=' + MediaOverviewType.QUERYTAG.value + '&medium_name=MAZ&tag=' + tag.name)
        today_str = datetime.datetime.today().strftime('%Y-%m-%d')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Testartikel', 'date_of_publication': today_str, 'text': 'Dies ist ein beispielhafter Beispieltext'}])


class TopMediaTest(CreateArticlesWithTagsTestCase):
    """
    This class contains test for the Top Media REST API
    defined in views.py > 'class TopMediaView(APIView)'.
    """

    def test_top_media_blank_api_wo_user(self):
        """
        Tests the API without a user specified in the query.
        Should return the article_count for every available medium ('MAZ') in the database.
        """
        logger.info("started TopMediaTest.test_top_media_blank_api_wo_user")
        response = self.client.get('/articles/top/media?mediaOverviewType=' + MediaOverviewType.ALLTAGS.value)
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'article_count': 1, 'medium_name': 'maz'}])

    def test_top_media_with_extra_articles(self):
        """
        Tests the API without a user specified in the query after two new articles are written to the database.
        Should return the article_count for every available medium ('MAZ', 'MOZ') in the database.
        """
        logger.info("started TopMediaTest.test_top_media_with_extra_articles")
        file = Files.objects.get(upload_file='testfile.pdf')
        tag_1 = Tag.objects.get(name="Test1")
        tag_2 = Tag.objects.get(name="Test2")
        input_data = {
            'medium_name': 'MOZ',
            'medium_type': 'Zeitung',
            'name': 'Testartikel2',
            'source_file': file,
            'reach': 1111,
            'date_of_publication': datetime.datetime.today(),
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': False,
            'tags': [tag_1.name]
        }
        write_article_to_db(**input_data)
        input_data['name'] = 'Testartikel3'
        input_data['tags'] = [tag_2.name]
        write_article_to_db(**input_data)
        response = self.client.get('/articles/top/media?mediaOverviewType=' + MediaOverviewType.ALLTAGS.value)
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'article_count': 2, 'medium_name': 'moz'}, {'article_count': 1, 'medium_name': 'maz'}])

    def test_top_media_with_extra_tag(self):
        """
        Tests whether the API handles user.favorite_tags correctly. User 1 is specified in the query.
        Writes a new article to the database without adding its tag to User 1s favorite_tags and tests the API response.
        The new article and its medium should not be taken into account in the response.
        Then, the new articles tag is added as a favorite_tag for User 1 and the API response is tested again.
        Now, the new article and its medium should be included in the API response.
        """
        logger.info("started TopMediaTest.test_top_media_with_extra_tag")
        file = Files.objects.get(upload_file='testfile.pdf')
        tag3_name = "Test3"
        input_data = {
            'medium_name': 'MOZ',
            'medium_type': 'Zeitung',
            'name': 'Testartikel2',
            'source_file': file,
            'reach': 1111,
            'date_of_publication': datetime.datetime.today(),
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': False,
            'tags': [tag3_name]
        }
        write_article_to_db(**input_data)
        response = self.client.get('/articles/top/media?mediaOverviewType=' + MediaOverviewType.USERTAGS.value + '&user=1')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'article_count': 1, 'medium_name': 'maz'}])
        user = User.objects.get(id_tag=1)
        tag3 = Tag.objects.get(name=tag3_name)
        user.favorite_tags.add(tag3)
        response = self.client.get('/articles/top/media?mediaOverviewType=' + MediaOverviewType.USERTAGS.value + '&user=1')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'article_count': 1, 'medium_name': 'maz'}, {'article_count': 1, 'medium_name': 'moz'}])

    def test_top_media_query_tag(self):
        """
        Tests the API with a specific tag ('Test1') passed as a query parameter.
        The API response should include article_counts for every medium that has articles tagged with that specific tag.
        """
        logger.info("started TopMediaTest.test_top_media_query_tag")
        response = self.client.get('/articles/top/media?mediaOverviewType=' + MediaOverviewType.QUERYTAG.value + '&tag=Test1')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'article_count': 1, 'medium_name': 'maz'}])

    def test_top_media_with_wrong_uid(self):
        """
        Tests how the API handles a request with a user id that doesn't exist.
        The API should return a response with status_code 400 (Bad Request).
        """
        logger.info("started TopMediaTest.test_top_media_with_wrong_uid")
        response = self.client.get('/articles/top/media?mediaOverviewType=' + MediaOverviewType.USERTAGS.value + '&user=2')
        self.assertEqual(response.status_code, 400)


class HotTopicTest(CreateArticlesWithTagsTestCase):
    """
    This class contains test for the Hot Topic REST API
    defined in views.py > 'class HotTopics(APIView)'.
    """

    def test_top_tag_blank(self):
        """
        Tests the API without a user specified in the query.
        The API should default to User 1 and return User 1s top tags ('Test1' and 'Test2', evenly split value of 0.5).
        """
        logger.info("started HotTopicTest.test_top_tag_blank")
        response = self.client.get('/articles/top/tags')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'article_count': 1, 'hotness_factor': 0.5, 'tag_name': 'Test1'},
                              {'article_count': 1, 'hotness_factor': 0.5, 'tag_name': 'Test2'}])

    def test_top_tag_user(self):
        """
        Tests the API with User 1 specified as a query parameter.
        The API should return User 1s top tags ('Test1' and 'Test2', evenly split value of 0.5).
        """
        logger.info("started HotTopicTest.test_top_tag_user")
        response = self.client.get('/articles/top/tags?user=1')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'article_count': 1, 'hotness_factor': 0.5, 'tag_name': 'Test1'},
                              {'article_count': 1, 'hotness_factor': 0.5, 'tag_name': 'Test2'}])

    def test_top_tag_wrong_uid(self):
        """
        Tests how the API handles a request with a user id that doesn't exist.
        The API should return a response with status_code 400 (Bad Request).
        """
        logger.info("started HotTopicTest.test_top_tag_wrong_uid")
        response = self.client.get('/articles/top/tags?user=2')
        self.assertEqual(response.status_code, 400)


class UserTagsTest(CreateArticlesWithTagsTestCase):
    """
    This class contains test for the User Favorite Tags REST API
    defined in views.py > 'class UserFavoriteTags(APIView)'.
    """

    def test_get_standard_user_tags(self):
        """
        Tests the API without a user specified in the query.
        The API should default to User 1 and return User 1s favorite tags ('Test1' and 'Test2').
        """
        logger.info("started UserTagsTest.test_get_standard_user_tags")
        response = self.client.get('/articles/user/tags')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Test1'}, {'name': 'Test2'}])

    def test_get_with_wrong_uid(self):
        """
        Tests how the API handles a request with a user id that doesn't exist.
        The API should return a response with status_code 400 (Bad Request).
        """
        logger.info("started UserTagsTest.test_get_with_wrong_uid")
        response = self.client.get('/articles/user/tags?user=2')
        self.assertEqual(response.status_code, 400)

    def test_get_new_user_tags(self):
        """
        Creates a new user and tests whether the API returns an empty array for that user.
        Then, a new tag ('Test3') is created and added to that users favorite_tags.
        The API should now return an array containing an object for tag 'Test3'.
        """
        logger.info("started UserTagsTest.test_get_new_user_tags")
        user = User(id_tag=14482)
        user.save()
        response = self.client.get('/articles/user/tags?user=14482')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [])
        tag = Tag(name="Test3")
        tag.save()
        user.favorite_tags.add(tag)
        article, _ = Article.objects.update_or_create(name='Testartikel')
        article.tags.add(tag)
        response = self.client.get('/articles/user/tags?user=14482')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Test3'}])

    def test_post_tags(self):
        """
        Test the APIs POST method to set favorite_tags for a specific user.
        First, the GET response is verified to both contain 'Test1' and 'Test2' as User 1s favorite_tags.
        Then, by sending a POST request, only 'Test1' is set as a favorite_tag.
        To verify that POST requests success, User 1s favorite_tags are retrieved again and should now only contain 'Test1'.
        """
        logger.info("started UserTagsTest.test_post_tags")
        response = self.client.get('/articles/user/tags')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Test1'}, {'name': 'Test2'}])
        response = self.client.post('/articles/user/tags', {'user': 1, 'tags': ['Test1']})
        self.assertEqual(response.status_code, 201)
        response = self.client.get('/articles/user/tags')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Test1'}])

    def test_post_no_user(self):
        """
        Tests how the API handles a POST request without a user specified in the body.
        The API should return a response with status_code 400 (Bad Request).
        """
        logger.info("started UserTagsTest.test_post_no_user")
        response = self.client.post('/articles/user/tags', {'tags': ['Test1']})
        self.assertEqual(response.status_code, 400)

    def test_post_no_tags(self):
        """
        Tests how the API handles a POST request without tags specified in the body.
        The API should return a response with status_code 400 (Bad Request).
        """
        logger.info("started UserTagsTest.test_post_no_tags")
        response = self.client.post('/articles/user/tags', {'user': 1})
        self.assertEqual(response.status_code, 400)

    def test_post_no_correct_user(self):
        """
        Tests how the API handles a POST request containing a user id that doesn't exist.
        The API should return a response with status_code 400 (Bad Request).
        """
        logger.info("started UserTagsTest.test_post_no_correct_user")
        response = self.client.post('/articles/user/tags', {'user': 1111, 'tags': ['Test1']})
        self.assertEqual(response.status_code, 400)


class TagsOverviewTest(CreateArticlesWithTagsTestCase):
    """ Tests how to get and set tags for users """

    def test_plain_api(self):
        """ tests api plain api """
        logger.info("started TagsOverviewTest.test_plain_api")
        response = self.client.get('/articles/tags/overview')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'name': 'Parent1', 'tags': [{'name': 'Test1'}, {'name': 'Test2'}]}])


class UserTagsRegionalInfluenceTest(CreateArticlesWithTagsTestCase):
    """ Tests the api that counts regionality of media per tag """

    def test_plain_api(self):
        """ tests api plain api """
        logger.info("started UserTagsRegionalInfluenceTest.test_plain_api")
        response = self.client.get('/articles/user/tags/regionality')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{"tag_name": "Test1", "count": 1, "regional_count": 1, "national_count": 0, "web_count": 0},
                             {"tag_name": "Test2", "count": 1, "regional_count": 1, "national_count": 0, "web_count": 0}])


class UserTagsArticleCountTest(CreateArticlesWithTagsTestCase):
    """ Tests api where user gets his tags with article counts """

    def test_plain_api(self):
        """ tests api plain api """
        logger.info("started UserTagsArticleCountTest.test_plain_api")
        response = self.client.get('/articles/user/tags/article_count')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'article_count': '1', 'topic_name': 'Test1'},
                              {'article_count': '1', 'topic_name': 'Test2'}])


class ArticleTest(CreateArticlesWithTagsTestCase):
    """ database test medium """
    reset_sequences = True

    def setUp(self):
        logger.info("started ArticleTest.setup")
        with mock.patch('articles.signals.extract_information_from_file', autospec=True) as mocked_post_save:
            post_save.connect(mocked_post_save, sender=Files, dispatch_uid='extract_information_from_file')
            MediumType.objects.create(name='my local', description='Lokale Zeitung')
            Medium.objects.create(name='JDs Zeitung', medium_type_id=1)
            Files.objects.create(upload_file='testfile.pdf', upload_date='18.11.2021')
            input_data = {
                'name': 'Testartikel',
                'date_of_publication': '1988-12-30',
                'reach': 1000,
                'pages': 2,
                'text': 'Dies ist ein beispielhafter Beispieltext',
                'wrongly_parsed': False,
            }
            Article.objects.create(medium_id=1, source_file_id=1, **input_data)

    def test_author(self):
        """ Tests get medium_type """
        logger.info("started test_author")
        article = Article.objects.get(name='Testartikel')
        self.assertEqual(article.date_of_publication, datetime.date(1988, 12, 30))
        self.assertEqual(article.medium.name, 'JDs Zeitung')

    def test_serializer(self):
        """ Tests the article serializer """
        logger.info("started ArticleTest.test_serializer")
        input_data = {
            'name': 'Testartikel2',
            'date_of_publication': '2021-12-30',
            'reach': 2000,
            'pages': 3,
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'wrongly_parsed': False,
            'source_file_id': 1,
        }
        medium = Medium.objects.get(name='JDs Zeitung')
        input_data_ = {'medium': medium, **input_data}
        ArticleSerializer().create(input_data_)
        self.assertEqual(Article.objects.count(), 2)


def add_medium_helper(medium_name_list):
    """ helps to perform tests for medium insertion """
    file = Files.objects.get(upload_file='testfile.pdf')
    input_data = {
        'medium_type': 'Zeitung',
        'name': 'Testartikel',
        'source_file': file,
        'reach': 1000,
        'date_of_publication': '1988-12-30',
        'text': 'Dies ist ein beispielhafter Beispieltext',
        'pages': 2,
        'wrongly_parsed': False,
        'tags': ['tag1', 'tag2']
    }

    for medium_name in medium_name_list:
        input_data['name'] = medium_name + " - Artikel"
        input_data['medium_name'] = medium_name
        write_article_to_db(**input_data)


class DbHandler(CreateArticlesWithTagsTestCase):
    """ Test the db integration"""
    reset_sequences = True

    def test_write_articles_name(self):
        """ test of method write_article_to_db field name (defined in CreateArticlesWithTagsTestCase)"""
        logger.info("started test_write_articles_name")
        article = Article.objects.get(name='Testartikel')
        self.assertEqual(article.reach, 1000)

    def test_write_articles_title(self):
        """ test of method write_article_to_db with field title"""
        logger.info("started test_write_articles_title")
        input_data = {
            'medium_name': 'MAZ',
            'medium_type': 'Zeitung',
            'title': 'Testartikel1',
            'source_file': self.file,
            'reach': 1000,
            'date_of_publication': '1988-12-30',
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': False,
            'tags': ['tag1', 'tag2']
        }
        write_article_to_db(**input_data)
        article = Article.objects.get(name='Testartikel1')
        self.assertEqual(article.reach, 1000)

    def test_write_articles_reach_string(self):
        """ test of method write_article_to_db with field reach as string """
        logger.info("started test_write_articles_reach_string")
        input_data = {
            'medium_name': 'MAZ',
            'medium_type': 'Zeitung',
            'title': 'Testartikel2',
            'source_file': self.file,
            'reach': '1.000',
            'date_of_publication': '1988-12-30',
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': False,
            'tags': ['tag1', 'tag2']
        }
        write_article_to_db(**input_data)
        article = Article.objects.get(name='Testartikel2')
        self.assertEqual(article.reach, 1000)

    def test_write_articles_reach_undefined_string(self):
        """ test of method write_article_to_db with field reach as string """
        logger.info("started test_write_articles_reach_string")
        input_data = {
            'medium_name': 'MAZ',
            'medium_type': 'Zeitung',
            'title': 'Testartikel3',
            'source_file': self.file,
            'reach': 'kA',
            'date_of_publication': '1988-12-30',
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': False,
            'tags': ['tag1', 'tag2']
        }
        write_article_to_db(**input_data)
        article = Article.objects.get(name='Testartikel3')
        self.assertEqual(article.reach, 0)

    def test_write_articles_medium_with_dash(self):
        """ test if medium and submedium will be merged """
        logger.info("started test_write_articles_reach_string")
        add_medium_helper(['Märkische Allgemeine', 'Märkische Allgemeine - Potsdamer'])
        self.assertEqual(Medium.objects.filter(name='märkische allgemeine').count(), 1)

        add_medium_helper(['Lausitzer Rundschau', 'Lausitzer Rundschau Cottbus'])
        self.assertEqual(Medium.objects.filter(name='lausitzer rundschau').count(), 1)
        self.assertEqual(Medium.objects.all().count(), 3)

        add_medium_helper(['Welt Am Sonntag', 'welt.de', 'welt online', 'dw', 'die welt'])
        self.assertEqual(Medium.objects.all().count(), 6)

        add_medium_helper(['nd', 'neues deutschland', 'nd (neues deutschland)'])
        self.assertEqual(Medium.objects.all().count(), 7)

        add_medium_helper(['wochenspiegel', 'wochenspiegel potsdam'])
        self.assertEqual(Medium.objects.all().count(), 8)

        add_medium_helper(['bild', 'bild.de', 'bildplus', 'bild der frau'])
        self.assertEqual(Medium.objects.all().count(), 10)

        add_medium_helper(['Der Spiegel', 'Spiegel+'])
        self.assertEqual(Medium.objects.all().count(), 11)

        add_medium_helper(['nordkurier', 'nordkurier potsdam'])
        self.assertEqual(Medium.objects.all().count(), 12)

        add_medium_helper(['schweriner volkszeitung', 'schweriner volkszeitung - Viertel'])
        self.assertEqual(Medium.objects.all().count(), 13)

        add_medium_helper(['frankfurter rundschau', 'frankfurter rundschau - Test'])
        self.assertEqual(Medium.objects.all().count(), 14)

        add_medium_helper(['nordwest-zeitung', 'nordwest-zeitung asd'])
        self.assertEqual(Medium.objects.all().count(), 15)
