""" Views with OpenAPI annotations """
import math
import datetime
import logging
import secrets

from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q, Count, Sum
from django.http import JsonResponse
from django.shortcuts import render
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.views import View
from django.views.generic import TemplateView

from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema
from rest_framework import generics
from rest_framework.response import Response
from rest_framework.views import APIView
import numpy as np
import numpy.polynomial.polynomial as poly
import pandas as pd
from plotly.offline import plot
import plotly.express as px

from one_api.settings_prod import MEDIA_ROOT
from articles.forms import FileUploadForm
from articles.models import Article, Timeline, Medium, MediumType, Overview, TopMedia, Tag, TopTags, Files, User, \
    ParentTag, GenFunction, FunctionData
from articles.serializers import ArticleSerializer, TimelineSerializer, MediumSerializer, MediumTypeSerializer, \
    OverviewSerializer, TopMediaSerializer, TagSerializer, TopTagsSerializer, TagOverviewSerializer, \
    TagArticleCountSerializer, RecentArticleSerializer, TagsRegionalInfluenceSerializer
from articles.tasks import read_all_media_files, generate_by_plot, generate_tag
from articles.bgr_tasks import autoremove_task
from articles.definitions import MediaRegionality, MediaOverviewType

logger = logging.getLogger(__name__)


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all articles"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new article or updates if the name is already in the db"
))
class ArticleList(generics.ListCreateAPIView):
    """ Lists all articles """
    queryset = Article.objects.all()
    serializer_class = ArticleSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one article selected by Id"
))
class ArticleDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Get details of article """
    queryset = Article.objects.all()
    serializer_class = ArticleSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all medias"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new medium or updates if the name is already in the db"
))
class MediumList(generics.ListCreateAPIView):
    """ List all media """
    queryset = Medium.objects.all()
    serializer_class = MediumSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one medium selected by Id"
))
class MediumDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Gets details of medium """
    queryset = Medium.objects.all()
    serializer_class = MediumSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all types of media"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new medium_type or updates if the name is already in the db"
))
class MediumTypeList(generics.ListCreateAPIView):
    """ List all medium_types """
    queryset = MediumType.objects.all()
    serializer_class = MediumTypeSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one medium_type selected by Id"
))
class MediumTypeDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Get details of medium_type """
    queryset = MediumType.objects.all()
    serializer_class = MediumTypeSerializer


def user_tags(request_params):
    """ get favorite tags by generated user id """
    id_tag = 1
    if request_params.__contains__('user'):
        id_tag = request_params['user']
    user = User.objects.get(id_tag=id_tag)
    tag_names = []
    for tag in user.favorite_tags.all():
        tag_names.append(tag.name)
    return tag_names


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows how the tags perform over time"
))
class TimelineList(generics.ListAPIView):
    """ List all timeline objects """
    today = datetime.datetime.today()
    datetime_recent = today - datetime.timedelta(days=365)
    queryset = Timeline.objects.filter(
        Q(year=today.year, month__lte=today.month) | Q(year=datetime_recent.year, month__gte=datetime_recent.month))
    serializer_class = TimelineSerializer


user_param_get = openapi.Parameter('user', openapi.IN_QUERY, description="id which is connected to the user",
                                   type=openapi.TYPE_NUMBER)


class OverviewList(APIView):
    """ List all overview objects """
    today = datetime.datetime.today()
    datetime_recent = today - datetime.timedelta(days=90)

    @swagger_auto_schema(manual_parameters=[user_param_get], responses={200: OverviewSerializer(many=True)})
    def get(self, request):
        """ get overview, a user can be selected with user=user_id """
        try:
            tag_names = user_tags(request.query_params)
            overview_objects = Overview.objects.filter(
                date_of_publication__range=[self.datetime_recent, self.today]).filter(name__in=tag_names)
            overview_serializer = OverviewSerializer(overview_objects, many=True)
            return JsonResponse(overview_serializer.data, safe=False)
        except ObjectDoesNotExist:
            logger.warning('User with id_tag %s does not exists', str(request.query_params['user']))
            return Response({'message': 'User does not exist.'}, 400)


class Home(TemplateView):
    """ Landing page """
    template_name = 'wizard/home.html'

    def get_context_data(self, **kwargs):
        """ generate context data for Landing Page"""
        context = super().get_context_data(**kwargs)
        context['site_urls'] = {
            'articles': [
                {'name': 'open-api', 'url': '/swagger/'},
                {'name': 'articles', 'url': '/articles/articles'},
                {'name': 'media', 'url': '/articles/media'},
                {'name': 'medium type', 'url': '/articles/mediumtype'},
                {'name': 'by Tags', 'url': '/articles/byTags'},
                {'name': 'overview', 'url': '/articles/overview'},
            ],
            'pr_upload': [
                {'name': 'upload a file', 'url': '/pr_upload'},
            ],
            'wizard': [
                {'name': 'Wizard Overview', 'url': '/wizard/overview'},
            ],
            'projekt': [
                {'name': 'Repository', 'url': 'https://gitup.uni-potsdam.de/innohs/rest-api'},
                {'name': 'Projektbeschreibung',
                 'url': 'https://www.uni-potsdam.de/de/multimedia/projekte/institutionelle-infrastrukturen/innovative-hochschule'},
            ],
        }
        return context


def create_function(raster_id):
    """ build the graph on the wizard overview """
    date_bounds = (datetime.date(2021, 12, 1), datetime.date(2022, 6, 30))
    study_date = datetime.date(2022, 3, 1)
    study_follow_date = datetime.date(2022, 4, 1)
    date_delta = date_bounds[1] - date_bounds[0]
    all_x = np.array(range(0, date_delta.days))

    x_data = [0, 0, 0, 0, 0]
    x_data[1] = (study_date - date_bounds[0]).days
    x_data[3] = (study_follow_date - date_bounds[0]).days
    x_data[2] = secrets.choice(list(range(x_data[1], x_data[3])))
    x_data[4] = x_data[1] + x_data[3]
    y_data = [0, 0, 0, 0, 0]
    y_data[0] = secrets.choice(list(range(1, 5)))
    y_data[1] = secrets.choice(list(range(5, 7)))
    y_data[2] = secrets.choice(list(range(2, 5)))
    y_data[3] = secrets.choice(list(range(6, 10)))
    y_data[4] = secrets.choice(list(range(8, 13)))
    series = poly.Polynomial.fit(x=x_data, y=y_data, deg=3)

    y_concrete = []
    for part_x in all_x:
        value = series(part_x)
        value = secrets.choice([0, math.floor(value), math.ceil(value)])
        value = max(0, value)
        y_concrete.append(value)

    x_data = list(series(all_x))
    g_func, _ = GenFunction.objects.update_or_create(raster_id=raster_id, date_delta=date_delta.days)
    for data in list(zip(x_data, y_concrete)):
        FunctionData.objects.update_or_create(x_data=data[0], y_data=data[1], raster=g_func)


def generate_plot(data, date_delta):
    """ drawing the plot from given function data """
    x_date_range = pd.date_range('2021-12-01', periods=date_delta, freq='D')
    dframe = pd.DataFrame(data, index=x_date_range, columns=['scenario', 'generated article'])
    fig = px.line(dframe)
    fig.update_traces(showlegend=False)
    return plot(fig, output_type='div')


def filter_by_regionality(media_regionality):
    """ receives media regionality enum and provides list of matching media outlets """
    outlets = []
    for media in Medium.objects.filter(medium_type__name=media_regionality.value).all():
        outlets.append(media.name)
    return outlets


class OverviewDashboard(TemplateView):
    """ Wizard pages """
    template_name = 'wizard/overview.html'

    def get_context_data(self, **kwargs):
        """ generate context data for overview Page"""
        article_count = Article.objects.count()
        tag_count = Tag.objects.count()

        wrongly_parsed_articles = Article.objects.filter(wrongly_parsed=True).all()
        wrongly_parsed_files = Files.objects.filter(article__wrongly_parsed=True).all()

        context = super().get_context_data(**kwargs)
        context['articles'] = {
            'count': article_count,
            'wrongly_parsed_articles': wrongly_parsed_articles,
            'wrongly_parsed_files': wrongly_parsed_files,
            'wrongly_parsed_count': wrongly_parsed_articles.count()
        }
        context['media'] = {
            'count': Medium.objects.count(),
            'top': TopMedia.objects.all(),
            'top_regional': [medium for medium in TopMedia.objects.all() if medium.name in filter_by_regionality(MediaRegionality.REGIONAL)],
            'top_national': [medium for medium in TopMedia.objects.all() if medium.name in filter_by_regionality(MediaRegionality.NATIONAL)],
            'top_website': [medium for medium in TopMedia.objects.all() if medium.name in filter_by_regionality(MediaRegionality.WEBSITE)],
        }
        context['tags'] = {
            'count': tag_count,
            'top': TopTags.objects.all().order_by("-count", "name"),
            'all': Tag.objects.all()
        }
        context['files'] = {
            'count': Files.objects.count()
        }
        context['users'] = {
            'count': User.objects.count()
        }
        context['wfunctions'] = {
            'all': GenFunction.objects.all()
        }
        if (article_count == 0) or (tag_count == 0):
            context['ratio'] = {'apt': 0, 'tpa': 0}
        else:
            context['ratio'] = {
                'apt': article_count / tag_count,
                'tpa': tag_count / article_count
            }

        # generate article creation functions for all tags
        if Tag.objects.count() == 0:
            generate_tag(0)
        tag_count = Tag.objects.count()
        if GenFunction.objects.count() < tag_count:
            for i in range(GenFunction.objects.count(), tag_count):
                create_function(i)

        # visualize function
        funcs = GenFunction.objects.all()
        fplots = []
        for func in funcs:
            qdata = FunctionData.objects.filter(raster=func.raster_id)
            x_data, y_data = [], []
            for qdat in qdata:
                x_data.append(qdat.x_data)
                y_data.append(qdat.y_data)
            fplots.append(generate_plot(list(zip(x_data, y_data)), func.date_delta))
        context['fplots'] = fplots
        context['splot'] = 0
        return context

    def post(self, request):
        """ generating graphs and articles on button press """
        if 'gGraph' in request.POST:
            FunctionData.objects.all().delete()
            GenFunction.objects.all().delete()
        elif 'gArticle' in request.POST:
            generate_by_plot.delay()
        elif 'gTag' in request.POST:
            generate_tag(Tag.objects.count())
        return render(request, self.template_name, self.get_context_data())

    def get(self, request, *args, **kwargs):
        context = self.get_context_data()
        if 'lGraph' in request.GET:
            context['splot'] = request.GET['lGraph']
        return render(request, self.template_name, context)


class Simulator(View):
    """ Wizard pages """
    template_name = 'wizard/simulator.html'
    selected_user = {}

    def get(self, request, *args, **kwargs):
        """ Answer to GET request """
        logger.debug("GET Simulator")
        return render(request, self.template_name, {})

    def post(self, request, *args, **kwargs):
        """ Answer to POST request """
        logger.info("POST Simulator")
        read_all_media_files.delay()
        autoremove_task.delay(MEDIA_ROOT)
        return render(request, self.template_name, {})


class UploadView(View):
    """ single file upload view """
    template_name = 'articles/files_form.html'
    form_class = FileUploadForm

    def get(self, request, *args, **kwargs):
        """ get of UploadView """
        logger.debug("GET Uploadview")
        form = self.form_class(initial={'file_name': 'no_file.pdf'})
        return render(request, self.template_name, {'documents': Files.objects.all(), 'form': form})

    def post(self, request, *args, **kwargs):
        """ post of UploadView """
        logger.info("POST Uploadview")
        form = self.form_class(request.POST)
        if form.is_valid():
            name_ = form.cleaned_data['file_name']

            logger.debug("debug %i", Files.objects.filter(upload_file=name_).count())
            if Files.objects.filter(upload_file=name_).count() == 0:
                file = Files(upload_file=name_, upload_date=timezone.now())
                file.save()
        return render(request, self.template_name, {'documents': Files.objects.all(), 'form': form})


class TopMediaView(APIView):
    """ gets for a specific user and his favorite tags the top media platforms """

    @swagger_auto_schema(manual_parameters=[user_param_get], responses={200: TopMediaSerializer(many=True)})
    def get(self, request):
        """ get method """
        try:
            media_overview_type = request.query_params['mediaOverviewType']
            if media_overview_type == MediaOverviewType.ALLTAGS.value:
                # Counts the number of articles in a medium regardless of user tags
                top_media = Medium.objects.filter(article__wrongly_parsed=False).annotate(
                    article_count=Count('article', distinct=True)).order_by('-article_count', 'name')
            elif media_overview_type == MediaOverviewType.USERTAGS.value:
                tag_names = user_tags(request.query_params)
                # Counts the number of articles in a medium depending on user tags
                top_media = Medium.objects.filter(article__tags__name__in=tag_names, article__wrongly_parsed=False).annotate(
                    article_count=Count('article', distinct=True)).order_by('-article_count', 'name')
            elif media_overview_type == MediaOverviewType.QUERYTAG.value:
                tag_names = [request.query_params['tag']]
                # Counts the number of articles in a medium depending on query tag
                top_media = Medium.objects.filter(article__tags__name__in=tag_names, article__wrongly_parsed=False).annotate(
                    article_count=Count('article', distinct=True)).order_by('-article_count', 'name')
            raw_media_data = []
            for medium in top_media.all():
                raw_media_data.append({'medium_name': medium.name, 'article_count': medium.article_count})
                if len(raw_media_data) > 4:
                    break
            top_media_serializer = TopMediaSerializer(raw_media_data, many=True)
            return JsonResponse(top_media_serializer.data, safe=False)
        except ObjectDoesNotExist:
            logger.warning('User with id_tag %s does not exists', str(request.query_params['user']))
            return Response({'message': 'User does not exist.'}, 400)


def unwanted_request(request, message):
    """ if there are missing attributes in the request data this function is taking care of logging """
    logger.warning(message)
    logger.warning(request.data)
    return Response({'message': message}, 400)


user_param_post = openapi.Parameter('user', openapi.IN_BODY, description="id which is connected to the user",
                                    type=openapi.TYPE_NUMBER)
tags_param_post = openapi.Parameter('tags', openapi.IN_BODY, description="id which is connected to the user",
                                    type=openapi.TYPE_ARRAY, items=openapi.Items(type=openapi.TYPE_STRING))


class UserFavoriteTags(APIView):
    """ view to get and set favorite tags from user """

    @swagger_auto_schema(manual_parameters=[user_param_get], responses={200: TagSerializer(many=True)})
    def get(self, request):
        """ get tags for specific user """
        try:
            tag_names = user_tags(request.query_params)
            tags = Tag.objects.filter(name__in=tag_names, article__wrongly_parsed=False)
            tag_serializer = TagSerializer(tags, many=True)
            return JsonResponse(tag_serializer.data, safe=False)
        except ObjectDoesNotExist:
            logger.warning('User with id_tag %s does not exists', str(request.query_params['user']))
            return Response({'message': 'User does not exist.'}, 400)

    # doesnt work yet
    @swagger_auto_schema(request_body=openapi.Schema(
        type=openapi.TYPE_OBJECT,
        properties={
            'user': openapi.Schema(type=openapi.TYPE_NUMBER, description='user_id'),
            'tags': openapi.Schema(type=openapi.TYPE_ARRAY, items=openapi.Items(type=openapi.TYPE_STRING),
                                   description='tags which should be saved'),
        }),
        responses={201: 'tags are set', 400: 'an parameter is wrongly set'})
    def post(self, request):
        """ safes all given tags to a specific user """
        if request.data.__contains__('user'):
            if request.data.__contains__('tags'):
                try:
                    user = User.objects.get(id_tag=request.data['user'])

                except ObjectDoesNotExist:
                    return unwanted_request(request, 'User not found')
                user.favorite_tags.remove(*user.favorite_tags.all())
                if isinstance(request.data['tags'], str):
                    sent_tags = request.data['tags'].split(', ')
                else:
                    sent_tags = request.data['tags']
                user.favorite_tags.add(*Tag.objects.filter(name__in=sent_tags).all())
                logger.info("Added tags.")
                logger.info(request.data)
                return Response({'message': 'Hello World!'}, 201)
            return unwanted_request(request, 'Couldn\'t update tags without tags')
        return unwanted_request(request, 'Couldn\'t update tags without user id')


class UserTagsArticleCount(APIView):
    """ view to get and set favorite tags from user """

    @swagger_auto_schema(manual_parameters=[user_param_get], responses={200: TagSerializer(many=True)})
    def get(self, request):
        """ get tags with article count for specific user """
        try:
            tag_names = user_tags(request.query_params)
            tags_with_article_count = Tag.objects.filter(name__in=tag_names, article__wrongly_parsed=False).annotate(
                article_count=Count('article', distinct=True)).order_by('-article_count', 'name')
            raw_tag_data = []
            for twa in tags_with_article_count.all():
                raw_tag_data.append({'article_count': twa.article_count, 'topic_name': twa.name})
            tag_serializer = TagArticleCountSerializer(raw_tag_data, many=True)
            return JsonResponse(tag_serializer.data, safe=False)
        except ObjectDoesNotExist:
            logger.warning('User with id_tag %s does not exists', str(request.query_params['user']))
            return Response({'message': 'User does not exist.'}, 400)


class HotTopics(APIView):
    """ this class gives the hot hot hot topics!!! """

    @swagger_auto_schema(responses={200: TopTagsSerializer(many=True)})
    def get(self, request):
        """ get method """
        try:
            today = datetime.datetime.today()
            datetime_recent = today - datetime.timedelta(days=60)
            tag_names = user_tags(request.query_params)
            tags = Tag.objects.filter(name__in=tag_names, article__date_of_publication__range=[datetime_recent, today], article__wrongly_parsed=False).annotate(
                article_count=Count('article', distinct=True)).order_by('-article_count', 'name')
            tag_sum = tags.aggregate(Sum('article_count'))
            top_tags = []
            for tag in tags.all():
                top_tags.append({'tag_name': tag.name, 'article_count': tag.article_count,
                                 'hotness_factor': tag.article_count / tag_sum['article_count__sum']})
            top_tags_serializer = TopTagsSerializer(top_tags, many=True)
            return JsonResponse(top_tags_serializer.data, safe=False)
        except ObjectDoesNotExist:
            logger.warning('User with id_tag %s does not exists', str(request.query_params['user']))
            return Response({'message': 'User does not exist.'}, 400)


class TagsOverview(APIView):
    """ Overview for tags grouped by parent tags """
    queryset = Tag.objects.all()
    serializer_class = TagSerializer

    @swagger_auto_schema(responses={200: TagOverviewSerializer(many=True)})
    def get(self, request):
        """ getter for tags grouped by parent tags """
        parent_tags = ParentTag.objects.all()
        grouped_by_parent = []
        for parent_tag in parent_tags:
            tags = Tag.objects.filter(parent__name=parent_tag, article__wrongly_parsed=False).distinct()
            if tags:
                grouped_by_parent.append(
                    {'name': parent_tag.name, 'tags': tags})
        tag_overview_serializer = TagOverviewSerializer(grouped_by_parent, many=True)
        return JsonResponse(tag_overview_serializer.data, safe=False)


class UserTagsRegionalInfluence(APIView):
    """ Class to see the influence of regional media on a topic """

    @swagger_auto_schema(responses={200: TagsRegionalInfluenceSerializer(many=True)})
    def get(self, request):
        """ Returns number of regional/national/web media per tag """
        try:
            tag_names = user_tags(request.query_params)
            regionality_per_tag = []
            for tag in tag_names:
                top_media = Medium.objects.filter(article__tags__name=tag).annotate(
                    article_count=Count('article', distinct=True))

                count, regional_count, national_count, web_count = 0, 0, 0, 0
                for medium in top_media:
                    medium_type = MediumType.objects.filter(id=medium.medium_type_id)[0]
                    count += medium.article_count
                    if medium_type.name == MediaRegionality.REGIONAL.value:
                        regional_count += medium.article_count
                    elif medium_type.name == MediaRegionality.NATIONAL.value:
                        national_count += medium.article_count
                    elif medium_type.name == MediaRegionality.WEBSITE.value:
                        web_count += medium.article_count
                regionality_per_tag.append(
                    {'tag_name': tag, 'count': count, 'regional_count': regional_count, 'national_count': national_count, 'web_count': web_count})
            return JsonResponse(TagsRegionalInfluenceSerializer(regionality_per_tag, many=True).data, safe=False)
        except ObjectDoesNotExist:
            logger.warning('User with id_tag %s does not exists', str(request.query_params['user']))
            return Response({'message': 'User does not exist.'}, 400)


class RecentArticlesForMedium(APIView):
    """ API to get the 5 most recent articles, matching a specific medium and specific tag(s)  """

    @swagger_auto_schema(responses={200: RecentArticleSerializer(many=True)})
    def get(self, request):
        """ Returns an array of articles """
        try:
            media_overview_type = request.query_params['mediaOverviewType']
            medium = request.query_params['medium_name']
            if media_overview_type == MediaOverviewType.ALLTAGS.value:
                articles = Article.objects.filter(medium__name=medium, wrongly_parsed=False).all().order_by('-date_of_publication', 'name')
            elif media_overview_type == MediaOverviewType.USERTAGS.value:
                tag_names = user_tags(request.query_params)
                articles = Article.objects.filter(medium__name=medium, tags__name__in=tag_names, wrongly_parsed=False).all().order_by('-date_of_publication', 'name')
            elif media_overview_type == MediaOverviewType.QUERYTAG.value:
                tag_names = [request.query_params['tag']]
                articles = Article.objects.filter(medium__name=medium, tags__name__in=tag_names, wrongly_parsed=False).all().order_by('-date_of_publication', 'name')

            recent_articles = []
            for article in articles.all():
                recent_articles.append({
                    'name': article.name,
                    'date_of_publication': article.date_of_publication,
                    'text': article.text
                })
                if len(recent_articles) > 4:
                    break
            articles_serializer = RecentArticleSerializer(recent_articles, many=True)
            return JsonResponse(articles_serializer.data, safe=False)
        except ObjectDoesNotExist:
            logger.warning('User with id_tag %s does not exists', str(request.query_params['user']))
            return Response({'message': 'User does not exist.'}, 400)
